(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "./expr", "./insert", "./util"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("./expr"), require("./insert"), require("./util"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.expr, global.insert, global.util);
    global.tables = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _expr, _insert, _util) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.tablesToSQL = tablesToSQL;
  _exports.tableOptionToSQL = tableOptionToSQL;
  _exports.tableToSQL = tableToSQL;
  _exports.unnestToSQL = unnestToSQL;

  function tableToSQL(tableInfo) {
    const {
      table,
      db,
      as,
      expr,
      schema
    } = tableInfo;
    const database = (0, _util.identifierToSql)(db);
    const schemaStr = (0, _util.identifierToSql)(schema);
    let tableName = table && (0, _util.identifierToSql)(table);
    if (expr && expr.type === 'values') tableName = `(${(0, _util.commonOptionConnector)('VALUES', _insert.valuesToSQL, expr.values)})`;
    if (expr && expr.type !== 'values') tableName = (0, _expr.exprToSQL)(expr);
    const str = [database, schemaStr, tableName].filter(_util.hasVal).join('.');
    if (as) return `${str} AS ${(0, _util.identifierToSql)(as)}`;
    return str;
  }

  function unnestToSQL(unnestExpr) {
    const {
      type,
      as,
      expr,
      with_offset: withOffset
    } = unnestExpr;
    const result = [(0, _util.toUpper)(type), `(${expr && (0, _expr.exprToSQL)(expr) || ' '})`, (0, _util.commonOptionConnector)('AS', _util.identifierToSql, as), (0, _util.commonOptionConnector)((0, _util.toUpper)(withOffset && withOffset.keyword), _util.identifierToSql, withOffset && withOffset.as)];
    return result.filter(_util.hasVal).join(' ');
  }
  /**
   * @param {Array} tables
   * @return {string}
   */


  function tablesToSQL(tables) {
    const {
      type
    } = tables;
    if ((0, _util.toUpper)(type) === 'UNNEST') return unnestToSQL(tables);
    const baseTable = tables[0];
    const clauses = [];
    if (baseTable.type === 'dual') return 'DUAL';
    clauses.push(tableToSQL(baseTable));

    for (let i = 1; i < tables.length; ++i) {
      const joinExpr = tables[i];
      const {
        on,
        using,
        join
      } = joinExpr;
      const str = [];
      str.push(join ? ` ${join}` : ',');
      str.push(tableToSQL(joinExpr));
      str.push((0, _util.commonOptionConnector)('ON', _expr.exprToSQL, on));
      if (using) str.push(`USING (${using.map(_util.identifierToSql).join(', ')})`);
      clauses.push(str.filter(_util.hasVal).join(' '));
    }

    return clauses.filter(_util.hasVal).join('');
  }

  function tableOptionToSQL(tableOption) {
    const {
      keyword,
      symbol,
      value
    } = tableOption;
    const sql = [keyword.toUpperCase()];
    if (symbol) sql.push(symbol);
    sql.push(value);
    return sql.join(' ');
  }
});